import numpy
import os, sys, pickle
from subprocess import Popen, PIPE
from harkpython import harkbasenode


class HarkNode(harkbasenode.HarkBaseNode):
    GUI_SCRIPT_NAME = 'plotQuickMusicSpecCodeKivyGui.py'

    def __init__(self):
        self.outputNames = ("OUTPUT",)
        self.outputTypes = ("vector_float",)

        script_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), self.GUI_SCRIPT_NAME)
        python_exec = sys.executable
        self.proc = Popen([python_exec, script_path], stdin=PIPE)

        self.spectrum = numpy.array([])
        self.windowlen = 100

        self.interval = 1

    def __del__(self):
        self.proc.kill()

    def update(self, data):
        pickle.dump(data, self.proc.stdin)

    def gendata(self, data):
        tmp = numpy.array(data)[numpy.newaxis, :]
        if tmp.sum() == 0:
            return

        if len(self.spectrum) == 0:
            self.spectrum = numpy.zeros(((self.windowlen - 1), len(data)))
        self.spectrum = numpy.r_[self.spectrum, tmp]

        if len(self.spectrum) > self.windowlen:
            self.spectrum = self.spectrum[-self.windowlen:, :]

        if self.count % self.interval == 0:
            self.update({'spectrum': self.spectrum, 'offset': self.count})

    def calculate(self):
        if self.count == 0:
            # set Parameters
            if (getattr(self, "WINDOW_LENGTH", None) is not None) and (isinstance(self.WINDOW_LENGTH, int)):
                self.windowlen = self.WINDOW_LENGTH
            if (getattr(self, "UPDATE_INTERVAL", None) is not None) and (isinstance(self.UPDATE_INTERVAL, int)):
                self.interval = self.UPDATE_INTERVAL
            # check input terminal
            if (getattr(self, "SPECTRUM", None) is None) or (not isinstance(self.SPECTRUM, numpy.ndarray)):
                try:
                    raise AttributeError("There is an error in the network file. The type of input data to the plotQuickMUSICSpecKivy node must be Vector<float>.")
                except AttributeError as e:
                    print(e)
        self.gendata(self.SPECTRUM)
        self.outputValues["OUTPUT"] = self.SPECTRUM
