import numpy
import os, sys, pickle
from subprocess import Popen, PIPE
from harkpython import harkbasenode


def cartesian2polar(xyz):
    """
    cartesian2polar(x, y, z)
    converts coordinate from xyz to r\theta\phi
    """
    x, y, z = xyz
    if z == 0:
        raise ValueError("cartesian " + str(xyz) + " z must be nonzero.")

    r = numpy.sqrt(sum(map(lambda x: x**2, [x, y, z])))
    theta = numpy.arccos(z/r)
    phi = numpy.arccos(x / numpy.sqrt(sum(map(lambda x: x**2, [x, y]))))
    if y < 0:
        phi *= -1

    return (r, theta, phi)


def r2d(r):
    return 180.0 / numpy.pi * r


class HarkNode(harkbasenode.HarkBaseNode):
    GUI_SCRIPT_NAME = 'plotQuickSourceCodeKivyGui.py'

    def __init__(self):
        self.outputNames = ("OUTPUT", )
        self.outputTypes = ("vector_source",)

        script_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), self.GUI_SCRIPT_NAME)
        python_exec = sys.executable
        self.proc = Popen([python_exec, script_path], stdin=PIPE)

        self.points = {}
        self.windowlen = 300

        self.interval = 50

    def __del__(self):
        self.proc.kill()

    def update(self, data):
        pickle.dump(data, self.proc.stdin)

    def gendata(self, data):
        for src in data:
            if 'x' in src:
                id = src["id"]
                xyz = src["x"]
                self.points.setdefault(id, [])
                self.points[id].append((self.count, r2d(cartesian2polar(xyz)[2])))

        if self.count % self.interval == 0:
            for id in list(self.points.keys()):
                pts = self.points[id]
                if pts[0][0] < self.count - 1000:
                    cdel = self.count - 1000 - pts[0][0]
                    del self.points[id][:cdel]

                    if len(self.points[id]) == 0:
                        del self.points[id]

            self.update({'points': self.points,
                         'trange': (self.count - self.windowlen, self.count)})

    def calculate(self):
        if self.count == 0:
            # set Parameters
            if (getattr(self, "WINDOW_LENGTH", None) is not None) and (isinstance(self.WINDOW_LENGTH, int)):
                self.windowlen = self.WINDOW_LENGTH
            if (getattr(self, "UPDATE_INTERVAL", None) is not None) and (isinstance(self.UPDATE_INTERVAL, int)):
                self.interval = self.UPDATE_INTERVAL
            # check input terminal
            if (getattr(self, "SOURCE", None) is None) or (not isinstance(self.SOURCE, list)):
                try:
                    raise AttributeError("There is an error in the network file. The type of input data to the plotQuickSourceKivy node must be Vector<Source>.")
                except AttributeError as e:
                    print(e)
        self.gendata(self.SOURCES)
        self.outputValues["OUTPUT"] = self.SOURCES
