import numpy
import os, sys, pickle
from subprocess import Popen, PIPE
from harkpython import harkbasenode


class HarkNode(harkbasenode.HarkBaseNode):
    GUI_SCRIPT_NAME = 'plotQuickSpecCodeKivyGui.py'

    def __init__(self):
        self.outputNames = ("OUTPUT",)
        self.outputTypes = ("matrix_complex",)

        script_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), self.GUI_SCRIPT_NAME)
        python_exec = sys.executable
        self.proc = Popen([python_exec, script_path], stdin=PIPE)

        # self.specgram = [numpy.array([])]
        self.specgram = []
        self.windowlen = 600

        self.interval = 10
        self.channel = 0

    def __del__(self):
        self.proc.kill()

    def update(self, data):
        pickle.dump(data, self.proc.stdin)

    def gendata(self, data):
        # construct spectrum data
        for i in range(len(self.specgram)):
            tmp = 20 * numpy.log10(numpy.array(numpy.abs(data[i][:])) + 1)[numpy.newaxis, :]

            if len(self.specgram[i]) == 0:
                self.specgram[i] = numpy.zeros(((self.windowlen - 1), len(data[i][:])))
            self.specgram[i] = numpy.r_[self.specgram[i], tmp]

            if len(self.specgram[i]) > self.windowlen:
                self.specgram[i] = self.specgram[i][len(self.specgram[i])-self.windowlen:]

        # update spectrogram
        if self.count % self.interval == 0:
            arr = numpy.array(self.specgram[self.channel])
            self.update({'spectrum': arr, 'offset': self.count})
            """
            for i in range(len(self.specgram)):
                arr = numpy.array(self.specgram[i])
                # arr = numpy.fliplr(arr)
                # arr = numpy.flipud(arr)
                self.update({'spectrum': arr, 'offset': self.count})
            """

    def calculate(self):
        if self.count == 0:
            # set Parameters
            if (getattr(self, "WINDOW_LENGTH", None) is not None) and (isinstance(self.WINDOW_LENGTH, int)):
                self.windowlen = self.WINDOW_LENGTH
            if (getattr(self, "UPDATE_INTERVAL", None) is not None) and (isinstance(self.UPDATE_INTERVAL, int)):
                self.interval = self.UPDATE_INTERVAL
            if (getattr(self, "SELECT_CHANNEL", None) is not None) and (isinstance(self.SELECT_CHANNEL, int)):
                self.channel = self.SELECT_CHANNEL
            # check input terminal
            if (getattr(self, "FFT", None) is None) or (not isinstance(self.FFT, numpy.ndarray)):
                try:
                    raise AttributeError("There is an error in the network file. The type of input data to the plotQuickSpecKivy node must be Vector<std::complex<float> >.")
                except AttributeError as e:
                    print(e)
            # check parameter range of SELECT_CHANNEL
            if (self.channel < 0) or (len(self.FFT) <= self.channel):
                try:
                    raise IndexError("The number of channels entered is {}, but the value specified in the SELECT_CHANNEL parameter is {}. The range of values ​​that can be set for the SELECT_CHANNEL parameter is between '0' and 'number of input channels - 1'.".format(len(self.FFT), self.channel))
                except IndexError as e:
                    print(e)
            # initialize array by input matrix size
            self.specgram = [numpy.array([]) for i in range(len(self.FFT))]
        self.gendata(self.FFT)
        self.outputValues["OUTPUT"] = self.FFT
