import numpy
import os, sys, pickle
import multiprocessing as mp
from subprocess import Popen, PIPE
from harkpython import harkbasenode


class HarkNode(harkbasenode.HarkBaseNode):
    GUI_SCRIPT_NAME = 'plotQuickWaveformCodeKivyGui.py'

    def __init__(self):
        self.outputNames = ("OUTPUT",)
        self.outputTypes = ("matrix_float",)

        script_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), self.GUI_SCRIPT_NAME)
        python_exec = sys.executable
        self.proc = Popen([python_exec, script_path], stdin=PIPE)

        self.showwave = None
        self.windowlen = 20000
        self.wavelen = 0
        self.cursorskip = 5000
        self.max = 100

        self.interval = 5

        self.running = True

    def __del__(self):
        self.proc.kill()

    def update(self, data):
        pickle.dump(data, self.proc.stdin)

    def gendata(self, data):
        wave = numpy.array(data)
        self.showwave = wave if self.showwave is None else numpy.concatenate((self.showwave, wave), axis=1)
        self.wavelen += wave.shape[1]

        if self.count % self.interval == 0 and self.showwave.shape[1] > self.windowlen:
            cursor = self.cursorskip * ((self.wavelen - self.windowlen) // self.cursorskip)
            self.showwave = self.showwave[:, -(self.wavelen - cursor):]

            self.max = max(self.max, int(self.showwave.max()) + 1)

            self.update({'wave':     self.showwave[:, :self.windowlen],  # self.showwave
                         'offset':   cursor,  # max(0, self.wavelen - self.windowlen),
                         'channels': self.showwave.shape[0],
                         'trange':   (cursor, cursor + self.windowlen),  # (self.wavelen - self.windowlen, self.wavelen),
                         'yrange':  (-self.max, self.max),
                         })

    def calculate(self):
        if self.count == 0:
            # set Parameters
            if (getattr(self, "WINDOW_LENGTH", None) is not None) and (isinstance(self.WINDOW_LENGTH, int)):
                self.windowlen = self.WINDOW_LENGTH
            if (getattr(self, "UPDATE_INTERVAL", None) is not None) and (isinstance(self.UPDATE_INTERVAL, int)):
                self.interval = self.UPDATE_INTERVAL
            if (getattr(self, "INITIAL_AMP_SCALE", None) is not None) and (isinstance(self.INITIAL_AMP_SCALE, int)):
                self.max = self.INITIAL_AMP_SCALE
            # check input terminal
            if (getattr(self, "WAVE", None) is None) or (not isinstance(self.WAVE, numpy.ndarray)):
                try:
                    raise AttributeError("There is an error in the network file. The type of input data to the plotQuickWaveformKivy node must be Matrix<float>.")
                except AttributeError as e:
                    print(e)
        self.gendata(self.WAVE)
        self.outputValues["OUTPUT"] = self.WAVE
